import cv2          # pip install opencv-python
import numpy as np
import matplotlib.pyplot as plt

img1 = cv2.imread(input('File name = '))     #   画像512×512ファイルの読み込み
plt.imshow(img1)
plt.show()

f_0 = np.sum(img1, axis=2)/3         #   モノクロ化

#
#　　　ウェーブレット解析結果を辞書型として保存
#
WaveletAnal = {}
WaveletAnal['0'] = {'f0': f_0}

n = 9
N = 2**n

#
#      レベルｊの解析結果の図示
#
def disp_map(j):
    Cj = WaveletAnal[f'{j}']['f0']     #   係数C_h,k
    DHj = WaveletAnal[f'{j}']['DHj']
    DVj = WaveletAnal[f'{j}']['DVj']
    DDj = WaveletAnal[f'{j}']['DDj']
    #
    #    画像を配列imgsに設定
    #    大きさ512×512を４分割して使用
    #
    imgs = np.full((512, 512, 3), 0.5)
    for h in range((2**(n-j))):
        for k in range((2**(n-j))):
            for ii in range((2**j)//2):       #   スケールに合わせて引き延ばし
                for jj in range((2**j)//2):
                    for kk in range(3):     #   赤、緑、青の３チャンネル
                        #
                        #      C_hkの値
                        #
                        imgs[512//2 + ((2**j)//2)*h + ii]\
                                    [512//2 + ((2**j)//2)*k + jj][kk] \
                            = Cj[h][k] / 255
                        #
                        #     DH, DV, DDは、値０を黒に設定
                        #
                        imgs[512//2 + ((2**j)//2)*h + ii][((2**j)//2)*k + jj][kk] \
                            = 0.0
                        imgs[((2**j)//2)*h + ii][(512//2) + \
                                                 ((2**j)//2)*k + jj][kk] \
                            = 0.0
                        imgs[((2**j)//2)*h + ii][((2**j)//2)*k + jj][kk] \
                            = 0.0
                    if DHj[h][k] > 0:    #   正ならば緑に設定
                        imgs[((2**j)//2)*h + ii][(512//2) + \
                                    ((2**j)//2)*k + jj][1] \
                            = DHj[h][k] / 255
                    elif DHj[h][k] < 0:  #   負ならば赤に設定
                        imgs[((2**j)//2)*h + ii][(512//2) +\
                                    ((2**j)//2)*k + jj][0] \
                            = -DHj[h][k] /255
                    if DVj[h][k] > 0:
                        imgs[512//2 + ((2**j)//2)*h + ii][((2**j)//2)*k + jj][1] \
                            = DVj[h][k] / 255
                    elif DVj[h][k] < 0:
                        imgs[512//2 + ((2**j)//2)*h + ii][((2**j)//2)*k + jj][0] \
                            = -DVj[h][k] /255
                    if DDj[h][k] > 0:
                        imgs[((2**j)//2)*h + ii][((2**j)//2)*k + jj][1] \
                            = DDj[h][k] / 255
                    elif DDj[h][k] < 0:
                        imgs[((2**j)//2)*h + ii][((2**j)//2)*k + jj][0] \
                            = -DDj[h][k] /255
                    

    plt.imshow(imgs)    #   画像配列imgsの表示
    plt.xticks([])
    plt.yticks([])
    plt.title(f'j = {j}')
    plt.show()

 
for j in range(1,n):
    #      レベルｊの解析
    print(f'Level {j} started.')
    Cj_1 = WaveletAnal[f'{j-1}']['f0']
    Cj = np.empty((2**(n-j), 2**(n-j)))
    DHj = np.empty((2**(n-j), 2**(n-j)))
    DVj = np.empty((2**(n-j), 2**(n-j)))
    DDj = np.empty((2**(n-j), 2**(n-j)))
    #
    #　　　　下降サンプリング
    #
    for h in range(2**(n-j)):
        for k in range(2**(n-j)):
            Cj[h][k] = (Cj_1[2*h][2*k] + Cj_1[2*h+1][2*k] +
                          Cj_1[2*h][2*k+1] + Cj_1[2*h+1][2*k+1]) / 4
            DHj[h][k] = (Cj_1[2*h][2*k] - Cj_1[2*h+1][2*k] +
                          Cj_1[2*h][2*k+1] - Cj_1[2*h+1][2*k+1]) / 4
            DVj[h][k] = (Cj_1[2*h][2*k] + Cj_1[2*h+1][2*k] -
                          Cj_1[2*h][2*k+1] - Cj_1[2*h+1][2*k+1]) / 4
            DDj[h][k] = (Cj_1[2*h][2*k] - Cj_1[2*h+1][2*k] -
                          Cj_1[2*h][2*k+1] + Cj_1[2*h+1][2*k+1]) / 4
    WaveletAnal[f'{j}'] = {'f0': Cj}
    WaveletAnal[f'{j}']['DHj'] = DHj
    WaveletAnal[f'{j}']['DVj'] = DVj
    WaveletAnal[f'{j}']['DDj'] = DDj

    disp_map(j)     #   解析結果の表示


while True:
    print(f'0 < j < {n}') 
    j = int(input('j = '))
    if j <=0 or j >= n:
        break
    disp_map(j)
             
